/*
    SDL_mixer:  An audio mixer library based on the SDL library
    Copyright (C) 1997-2009 Sam Lantinga

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Sam Lantinga
    slouken@libsdl.org
*/
module org.libsdl.mixer;

import org.libsdl.audio;
import org.libsdl.endian;
import org.libsdl.error;
import org.libsdl.rwops;
import org.libsdl.stdinc;
import org.libsdl.versions;

immutable uint SDL_MIXER_MAJOR_VERSION = 1;
immutable uint SDL_MIXER_MINOR_VERSION = 2;
immutable uint SDL_MIXER_PATCHLEVEL    = 11;

void SDL_MIXER_VERSION(ref SDL_version ver)
{
	ver.major = SDL_MIXER_MAJOR_VERSION;
	ver.minor = SDL_MIXER_MINOR_VERSION;
	ver.patch = SDL_MIXER_PATCHLEVEL;
}

immutable uint MIX_MAJOR_VERSION = SDL_MIXER_MAJOR_VERSION;
immutable uint MIX_MINOR_VERSION = SDL_MIXER_MINOR_VERSION;
immutable uint MIX_PATCHLEVEL    = SDL_MIXER_PATCHLEVEL;

alias SDL_MIXER_VERSION MIX_VERSION;

extern (C)
{
	SDL_version* Mix_Linked_Version();
}

enum MIX_InitFlags : uint
{
	MIX_INIT_FLAC = 0x00000001,
	MIX_INIT_MOD  = 0x00000002,
	MIX_INIT_MP3  = 0x00000004,
	MIX_INIT_OGG  = 0x00000008
}

alias MIX_InitFlags.MIX_INIT_FLAC MIX_INIT_FLAC;
alias MIX_InitFlags.MIX_INIT_MOD MIX_INIT_MOD;
alias MIX_InitFlags.MIX_INIT_MP3 MIX_INIT_MP3;
alias MIX_InitFlags.MIX_INIT_OGG MIX_INIT_OGG;

extern (C)
{
	int Mix_Init(int flags);
	void Mix_Quit();
}

immutable uint MIX_CHANNELS = 8;

immutable uint MIX_DEFAULT_FREQUENCY = 22050;
static if (SDL_BYTEORDER == SDL_LIL_ENDIAN)
{
	immutable uint MIX_DEFAULT_FORMAT = AUDIO_S16LSB;
}
else
{
	immutable uint MIX_DEFAULT_FORMAT = AUDIO_S16MSB;
}
immutable uint MIX_DEFAULT_CHANNELS = 2;
immutable uint MIX_MAX_VOLUME = 128;

struct Mix_Chunk
{
	int allocated;
	Uint8* abuf;
	Uint32 alen;
	Uint8 volume;
}

enum Mix_Fading
{
	MIX_NO_FADING,
	MIX_FADING_OUT,
	MIX_FADING_IN
}

alias Mix_Fading.MIX_NO_FADING MIX_NO_FADING;
alias Mix_Fading.MIX_FADING_OUT MIX_FADING_OUT;
alias Mix_Fading.MIX_FADING_IN MIX_FADING_IN;

enum Mix_MusicType
{
	MUS_NONE,
	MUS_CMD,
	MUS_WAV,
	MUS_MOD,
	MUS_MID,
	MUS_OGG,
	MUS_MP3,
	MUS_MP3_MAD,
	MUS_FLAC
}

alias Mix_MusicType.MUS_NONE MUS_NONE;
alias Mix_MusicType.MUS_CMD MUS_CMD;
alias Mix_MusicType.MUS_WAV MUS_WAV;
alias Mix_MusicType.MUS_MOD MUS_MOD;
alias Mix_MusicType.MUS_MID MUS_MID;
alias Mix_MusicType.MUS_OGG MUS_OGG;
alias Mix_MusicType.MUS_MP3 MUS_MP3;
alias Mix_MusicType.MUS_MP3_MAD MUS_MP3_MAD;
alias Mix_MusicType.MUS_FLAC MUS_FLAC;

struct Mix_Music;

extern (C)
{
	int Mix_OpenAudio(int frequency, Uint16 format, int channels, int chunksize);
	int Mix_AllocateChannels(int numchans);
	int Mix_QuerySpec(int* frequency, Uint16* format, int* channels);
	Mix_Chunk* Mix_LoadWAV_RW(SDL_RWops* src, int freesrc);
}

Mix_Chunk* MixLoadWAV(const char* file)
{
	return Mix_LoadWAV_RW(SDL_RWFromFile(file, "rb\0".ptr), 1);
}

extern (C)
{
	Mix_Music* Mix_LoadMUS(const(char)* file);
	Mix_Music* Mix_LoadMUS_RW(SDL_RWops* rw);
	Mix_Chunk* Mix_QuickLoad_WAV(Uint8* mem);
	Mix_Chunk* Mix_QuickLoad_RAW(Uint8* mem, Uint32 len);
	void Mix_FreeChunk(Mix_Chunk* chunk);
	void Mix_FreeMusic(Mix_Music* music);
	int Mix_GetNumChunkDecoders();
	const(char)* Mix_GetChunkDecoder(int index);
	int Mix_GetNumMusicDecoders();
	const(char)* Mix_GetMusicDecoder(int index);
	Mix_MusicType Mix_GetMusicType(const(Mix_Music)* music);
	void Mix_SetPostMix(void function(void* udata, Uint8* stream, int len) mix_func, void* arg);
	void Mix_HookMusic(void function(void* udata, Uint8* stream, int len) mix_func, void* arg);
	void Mix_HookMusicFinished(void function() music_finished);
	void* Mix_GetMusicHookData();
	void Mix_ChannelFinished(void function(int channel) channel_finished);
}

immutable int MIX_CHANNEL_POST = -2;

extern (C)
{
	alias void function(int chan, void* stream, int len, void* udata) Mix_EffectFunc_t;
	alias void function(int chan, void* udata) Mix_EffectDone_t;
	int Mix_RegisterEffect(int chan, Mix_EffectFunc_t f, Mix_EffectDone_t d, void* arg);
	int Mix_UnregisterEffect(int channel, Mix_EffectFunc_t f);
	int Mix_UnregisterAllEffects(int channel);
}

immutable string MIX_EFFECTSMAXSPEED = "MIX_EFFECTSMAXSPEED";

extern (C)
{
	int Mix_SetPanning(int channel, Uint8 left, Uint8 right);
	int Mix_SetPosition(int channel, Sint16 angle, Uint8 distance);
	int Mix_SetDistance(int channel, Uint8 distance);
	int Mix_SetReverseStereo(int channel, int flip);
	int Mix_ReserveChannels(int num);
	int Mix_GroupChannel(int which, int tag);
	int Mix_GroupChannels(int from, int to, int tag);
	int Mix_GroupAvailable(int tag);
	int Mix_GroupCount(int tag);
	int Mix_GroupOldest(int tag);
	int Mix_GroupNewer(int tag);
	int Mix_PlayChannelTimed(int channel, Mix_Chunk* chunk, int loops, int ticks);
}

int Mix_PlayChannel(int channel, Mix_Chunk* chunk, int loops)
{
	return Mix_PlayChannelTimed(channel, chunk, loops, -1);
}

extern (C)
{
	int Mix_PlayMusic(Mix_Music* music, int loops);
	int Mix_FadeInMusic(Mix_Music* music, int loops, int ms);
	int Mix_FadeInMusicPos(Mix_Music* music, int loops, int ms, double position);
	int Mix_FadeInChannelTimed(int channel, Mix_Chunk* chunk, int loops, int ms, int ticks);
}

int Mix_FadeInChannel(int channel, Mix_Chunk* chunk, int loops, int ms)
{
	return Mix_FadeInChannelTimed(channel, chunk, loops, ms, -1);
}

extern (C)
{
	int Mix_Volume(int channel, int volume);
	int Mix_VolumeChunk(Mix_Chunk* chunk, int volume);
	int Mix_VolumeMusic(int volume);
	int Mix_HaltChannel(int channel);
	int Mix_HaltGroup(int tag);
	int Mix_HaltMusic();
	int Mix_ExpireChannel(int channel, int ticks);
	int Mix_FadeOutChannel(int which, int ms);
	int Mix_FadeOutGroup(int tag, int ms);
	int Mix_FadeOutMusic(int ms);
	Mix_Fading Mix_FadingMusic();
	Mix_Fading Mix_FadingChannel(int which);
	void Mix_Pause(int channel);
	void Mix_Resume(int channel);
	int Mix_Paused(int channel);
	void Mix_PauseMusic();
	void Mix_ResumeMusic();
	void Mix_RewindMusic();
	int Mix_PausedMusic();
	int Mix_SetMusicPosition(double position);
	int Mix_Playing(int channel);
	int Mix_PlayingMusic();
	int Mix_SetMusicCMD(const(char)* command);
	int Mix_SetSynchroValue(int value);
	int Mix_GetSynchroValue();
	Mix_Chunk* Mix_GetChunk(int channel);
	void Mix_CloseAudio();
}

alias SDL_SetError Mix_SetError;
alias SDL_GetError Mix_GetError;

